-- KillCounterCommand.lua
--
-- KillCounter command line handler
--
-- Globals: KillCounter, setglobal


local KC = KillCounter;
local debugEnabled;
KC:EnableDebug(function(e) debugEnabled = e; end);

local TOGGLES = {
    {
        name = "NoTooltip",
        [true] = { "Tooltip messages disabled.", "notooltip" },
        [false] ={ "Tooltip messages enabled.", "tooltip" };
    };

    {
        name = "TerseTooltip",
        [true] = { "Only terse tooltip information.", "tersetip" },
        [false] = { "Full tooltip information.", "verbosetip" };
    };

    {
        name = "ShowGuid",
        [true] = { "GUID shown in tooltip.", "showguid" },
        [false] = { "GUID not shown in tooltip.", "noguid" };
    };

    {
        name = "Quiet",
        [true] = { "Kill bell suppressed.", "quiet" },
        [false] = { "Kills ring a bell.", "bell" };
    };

    {
        name = "BellStack",
        [true] = { "Rapid kills stack the bell.", "bellstack" },
        [false] = { "Rapid kills ring bell once.", "onebell" };
    };
};

local TOGGLE_KEYS = {};
for _, config in ipairs(TOGGLES) do
    local name = config.name;

    local onCommand = config[true][2];
    local offCommand = config[false][2];
    TOGGLE_KEYS[onCommand] = config;
    TOGGLE_KEYS[offCommand] = config;
end

function KC:Command_OptionStatus()
    self:Out("Current settings:");
    local command = self.slashCommands[1];

    local options = self.options;
    for _, config in ipairs(TOGGLES) do
        local curState, otherState;
        if (options[config.name]) then
            curState = config[true];
            otherState = config[false];
        else
            curState = config[false];
            otherState = config[true];
        end

        self:BulkOut(curState[1],
                     "(Toggle with",
                     command,
                     otherState[2] .. ")");
    end
end

function KC:Command_RecentKills()
    self:Out("Recent kills:");

    local cache = self.dataCache;

    if (cache) then
        local HANDLE_LIST = {};
        for handle, data in pairs(cache) do
            if (data.recent and data.recent > 0) then
                table.insert(HANDLE_LIST, handle);
            end
        end

        table.sort(HANDLE_LIST, function(a, b)
                                    return cache[a].recent < cache[b].recent;
                                end);


        for _, handle in ipairs(HANDLE_LIST) do
            local name = self:LookupNameByHandle(handle) or handle;
            local data = self.dataCache[handle];
            self:BulkOut(name, " ", tostring(data.recent));
        end
    end

    self:BulkOut("TOTAL ", tostring(self.recentCount or 0));
end

function KC:Command(msg)
    if (msg == "") then
        local command = self.slashCommands[1];
        self:Command_OptionStatus();
        self:Out("Use", command, "kills to see recent kill details",
                 "(" .. (self.recentCount or 0), "recorded)");
        return;
    end

    local toggle = TOGGLE_KEYS[msg];
    if (toggle) then
        local newValue = (toggle[true][2] == msg) or false;
        self:SetOption(toggle.name, newValue);
        self:Out(toggle[newValue][1]);
        return;
    end

    local count = msg:match("^ring%s([0-9]+)$");
    if (count) then
        KC:RingBell(tonumber(count));
        return;
    end

    if (msg == "kills") then
        KC:Command_RecentKills();
        return;
    end

    KC:Out("Unknown command '" .. msg .. "'");
end

-- Initialize commands
SlashCmdList[KC.slashCommandId] = function(...) return KC:Command(...) end

for i, v in ipairs(KC.slashCommands) do
    setglobal("SLASH_" .. KC.slashCommandId .. i, v);
end