-- KillCounterData.lua
--
-- Data access / summaries
--
-- Globals: KillCounter

local KC = KillCounter;

local select = select;

local RECORD = {};
local RECORD_META = { __index = RECORD; }

function KC:GetRecord(handle, createIfMissing, name)
    local data = self:GetDatabaseData(handle, createIfMissing, name);
    if (data == nil) then
        return;
    end

    local record = { data = data; handle = handle; db = self }
    setmetatable(record, RECORD_META);
    return record;
end

function RECORD:Free()
    self.data = nil;
end

function RECORD:AddKill(assoc, level, count, notRecent)
    local db = self.db;
    db:AddKillData(self.data, assoc, level, count, notRecent);
    db:SaveDatabaseData(self.handle);
end

function RECORD:GetTotalKills(level)
    local data = self.data;
    local total = data.total;
    if (level == nil) then
        return total, data.recent;
    end
    local ldata = data[level];
    if (ldata == nil) then
        return total, data.recent, 0;
    end
    return total, data.recent, ldata[1];
end

local SUMMARY_SUFFIX = {
    ["N"] = " NPCs",
    ["O"] = " other players",
    ["ON"] = " with other players and NPCs"
}

KC.SUMMARY_KEYS = {};
local SUMMARY_MAP = {};

local function AddSummaryType(label, ...)
    table.insert(KC.SUMMARY_KEYS, label);
    for i = 1, select('#', ...) do
        SUMMARY_MAP[select(i, ...)] = label;
    end
end

AddSummaryType("Kills", 1);
AddSummaryType("Solo", "");
AddSummaryType("With group", "G", "R", "PG", "PR");
AddSummaryType("With group and NPCs", "GN", "RN", "PGN", "PRN");
AddSummaryType("With group and others", "GO", "RO", "PGO", "PRO");
AddSummaryType("With group, others, and NPCs", "GON", "RON", "PGON", "PRON");
AddSummaryType("With NPCs", "N");
AddSummaryType("With others", "O");
AddSummaryType("With others, and NPCs", "ON");
AddSummaryType("Group only", "XG", "XR");
AddSummaryType("Group and NPCs only", "XGN", "XRN");
AddSummaryType("Group and others only", "XGO", "XRO");
AddSummaryType("Group, others, and NPCs only", "XGON", "XRON");
AddSummaryType("Pet only", "P");
AddSummaryType("Pet and NPCs only", "PN");
AddSummaryType("Pet and others only", "PO");
AddSummaryType("Pet, others, and NPCs only", "PON");
AddSummaryType("Imported", "I");

function RECORD:GetSummary(level)
    local data = self.data;
    local summary = { ["Kills"] = { "Kills", 0, 0, data.recent } };
    for dlevel, ldata in pairs(data) do
        if (type(dlevel) == "number") then
            for k, v in pairs(ldata) do
                local key = SUMMARY_MAP[k];
                if (key == nil) then
                    key = "<" .. k .. ">";
                    table.insert(KC.SUMMARY_KEYS, key);
                    SUMMARY_MAP[k] = key;
                end
                local cur = summary[key];
                if (cur == nil) then
                    -- LABEL, TOTAL, LEVEL, [SESSION]
                    cur = { key, 0, 0 }
                    summary[key] = cur;
                end
                cur[2] = cur[2] + v;
                if (dlevel == level) then
                    cur[3] = cur[3] + v;
                end
            end
        end
    end
    for index, key in ipairs(KC.SUMMARY_KEYS) do
        local val = summary[key];
        if (val) then
            summary[key] = nil;
            table.insert(summary, val);
        end
    end
    return summary;
end

