-- KillCounterDatabase
--
-- Globals: KillCounter, setglobal

local KC = KillCounter;
local debugEnabled;
KC:EnableDebug(function(e) debugEnabled = e; end);

KC.dataCache = {};

if (type(KC.playerNameDB) ~= "table") then
    KC.playerGuidDB = {};
end

if (type(KC.handleNameDB) ~= "table") then
    KC.handleNameDB = {};
end

function KC:GetHandle(guid)
    local guidType = guid:sub(5,5);
    if (guidType == "3") then
        return guid:sub(6,12);
    end
    -- "4" is pet
    if (guidType == "0") then
        return guid;
    end
end

local function getDB(self)
    local DB = self.dbReference;
    if (DB == nil) then
        DB = getglobal(self.playerDB);
        if (type(DB) ~= "table") then
            DB = {};
            setglobal(self.playerDB, DB);
        end
        if (type(DB.byHandle) ~= "table") then
            DB.byHandle = {};
        end
        self.dbReference = DB;
    end
    return DB;
end

local function DecodeData(enc, name)
    local data = {
        total = 0;
        maxLevel = 0;
        recent = 0;
        name = name;
    }

    while (enc ~= "") do
        local level, lenc, rest = enc:match("^([0-9]+):([^;]*);?(.*)$");
        if (not level) then
            break;
        end
        enc = rest;
        level = tonumber(level);
        local ldata = { [1] = 0; }
        local ltotal = 0;
        while (lenc ~= "") do
            local ak,av,bk,bv,ck,cv,rest =
                lenc:match("^([A-Z]*)([0-9]+)([A-Z]*)([0-9]*)([A-Z]*)([0-9]*)(.*)");
            if (not av) then
                break;
            end
            lenc = rest;
            av = tonumber(av);
            bv = tonumber(bv);
            cv = tonumber(cv);
            if (av) then
                ltotal = ltotal + av;
                ldata[ak] = (ldata[ak] or 0) + av;
            end
            if (bv) then
                ltotal = ltotal + bv;
                ldata[bk] = (ldata[bk] or 0) + bv;
            end
            if (cv) then
                ltotal = ltotal + cv;
                ldata[ck] = (ldata[ck] or 0) + cv;
            end
        end

        if (ltotal > 0) then
            ldata[1] = ltotal;
            data[level] = ldata;
            data.total = data.total + ltotal;
            if (level > data.maxLevel) then
                data.maxLevel = level;
            end
        end
    end

    return data;
end

function KC:AddKillData(data, assoc, level, count, notRecent)
    count = count or 1;
    local ldata = data[level];
    if (ldata == nil) then
        ldata = {
            [1] = count;
        }
        data[level] = ldata;
        if (level > data.maxLevel) then
            data.maxLevel = level;
        end
    else
        ldata[1] = ldata[1] + count;
    end
    ldata[assoc] = (ldata[assoc] or 0) + count;
    data.total = data.total + count;
    if (not notRecent) then
        data.recent = data.recent + count;
        self.recentCount = (self.recentCount or 0) + count;
    end
end

local function ApplyLegacyData(self, handle, data, legacyData)
    for level, kills in pairs(legacyData) do
        self:AddKillData(data, "I", level, kills, true);
    end
    self:SaveDatabaseData(handle);
end

function KC:GetDatabaseData(handle, createIfMissing, name)
    local data = self.dataCache[handle];
    if (data ~= nil) then
        return data;
    end
    -- Verify the name
    local _, newName;
    name, _, newName = self:CheckNameForHandle(handle, name);

    local DB = getDB(self);
    local encData = DB.byHandle[handle];
    local legacyData;
    if (newName) then
        legacyData = self:LegacyImport(handle, name);
    end
    if (encData == nil) then
        if (not createIfMissing and not legacyData) then
            return nil;
        end
        encData = "";
    end
    if (debugEnabled) then
        self:Debug("Decoding data for", handle);
    end
    data = DecodeData(encData, name);
    self.dataCache[handle] = data;
    if (legacyData) then
        ApplyLegacyData(self, handle,  data, legacyData);
    end
    return data;
end

function KC:SaveDatabaseData(handle)
    if (debugEnabled) then
        self:Debug("Attempting to save", handle);
    end
    local data = self.dataCache[handle];
    if (data == nil) then
        self:Info("ARGH! No data for", handle);
        return;
    end
    local enc = "";
    local KEYS = self.CODE_LIST;
    for l = 1, data.maxLevel do
        local ldata = data[l];
        if (ldata) then
            local lenc = l .. ":";
            local anydata = nil;
            for _, k in ipairs(KEYS) do
                local v = tonumber(ldata[k]);
                if (v) then
                    lenc = lenc .. k .. v;
                    anydata = true;
                end
            end
            if (anydata) then
                if (enc == "") then
                    enc = lenc;
                else
                    enc = enc .. ";" .. lenc;
                end
            end
        end
    end

    local DB = getDB(self);
    if (enc == "") then
        DB.byHandle[handle] = nil;
    else
        DB.byHandle[handle] = enc;
    end
end

---------------------------------------------------------------------------
function KC:CheckNameForHandle(handle, name)
    local curName = self:LookupNameByHandle(handle);
    if (curName) then
        if (name == nil or curName == name) then
            return curName;
        end
    elseif (not name) then
        return;
    end
    local notPlayer = self:SaveNameByHandle(handle, name);
    return name, true, notPlayer;
end

function KC:LookupNameByHandle(handle)
    local name = self.handleNameDB[handle];
    if (name) then
        return name;
    end
    return self.playerGuidDB[handle];
end

function KC:SaveNameByHandle(handle, name)
    if (debugEnabled) then
        self:Debug("Saving name", name, "for", handle);
    end
    if (handle:sub(1, 2) == "0x") then
        self.playerGuidDB[handle] = name;
        return false;
    end
    self.handleNameDB[handle] = name;
    return true;
end

---------------------------------------------------------------------------
function KC:InitSavedData()
    local globalDB = getglobal(self.globalDB);
    if (type(globalDB) ~= "table") then
        globalDB = {};
        setglobal(self.globalDB, globalDB);
        -- Save default options
        self:SaveOptions();
    end

    local playerDB = getglobal(self.playerDB);
    if (type(playerDB) ~= "table") then
        playerDB = {};
        setglobal(self.playerDB, playerDB);
    end
end
