-- KillCounterEvent.lua
--
-- Event management / timer support
--
-- Globals: KillCounter, PlaySound

local KC = KillCounter;
local debugEnabled;
KC:EnableDebug(function(e) debugEnabled = e; end);

local function KC_OnEvent(self, event, ...)
    if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
        KC:OnCombatLogEvent(...);
        return;
    end
    if (event == "ADDON_LOADED") then
        local addon = ...;
        if (addon == KC.name) then
            KC:AddonLoaded();
        end
    end
end

local RING_INTERVAL = 2.5;
local pendingRings = 0;
local nextEvent = nil;

local function FireEvent()
    if (pendingRings <= 0) then
        pendingRings = 0;
        return;
    end

    if (debugEnabled) then
        KC:Debug("Ringing bell at", tostring(GetTime()));
    end

    PlaySound("AuctionWindowOpen");
    pendingRings = pendingRings - 1;
    nextEvent = RING_INTERVAL;
end

local function KC_OnUpdate(self, elapsed)
    if (nextEvent) then
        nextEvent = nextEvent - elapsed;
        if (nextEvent > 0) then
            return;
        end
        nextEvent = nil;
        FireEvent();
        if (nextEvent) then
            return;
        end
    end
    if (debugEnabled) then
        KC:Debug("Hiding OnUpdate frame at", tostring(GetTime()));
    end
    self:Hide();
end

local eventFrame = CreateFrame("Frame");
eventFrame:SetScript("OnEvent", KC_OnEvent);
eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
eventFrame:RegisterEvent("ADDON_LOADED");
eventFrame:SetScript("OnUpdate", KC_OnUpdate);

function KC:RingBell(count)
    if (self.options.Quiet) then
        return;
    end
    if (not self.options.BellStack) then
        if (nextEvent) then
            return;
        end
        count = 1;
    end
    pendingRings = pendingRings + count;
    if (not nextEvent) then
        FireEvent();
    end
    eventFrame:Show();
end