-- KillCounterLegacy.lua
--
-- Code to import legacy KillCounter data
--
-- Globals: KillCounter, UnitName, GetRealmName

local KC = KillCounter;
local debugEnabled;
KC:EnableDebug(function(e) debugEnabled = e; end);

function KC:GetLegacyPlayerData()
    local db = getglobal(self.globalDB);

    if (not (db and db.data)) then
        return;
    end
    local data = db.data[GetRealmName()];
    if (data == nil) then
        return;
    end
    local pdata = data[UnitName("player")];
    if (pdata == nil) then
        return;
    end
    return pdata, data;
end

function KC:PrepLegacy()
    local pdata, data = self:GetLegacyPlayerData();
    if (pdata == nil) then
        return;
    end

    local toImport = {};
    local count = 0;
    for level, ldata in pairs(pdata) do
        if (type(ldata) == "table") then
            for name, _ in pairs(ldata) do
                toImport[name] = 1;
                count = count + 1;
            end
        end
    end

    if (next(toImport)) then
        KC.toImport = toImport;
        KC:Info("Scanned legacy database for potential imports: ",
                tostring(count), "found");

    else
        -- Discard this player's data
        data[UnitName("player")] = nil;
    end
end

function KC:LegacyImport(handle, name)
    if (debugEnabled) then
        self:Debug("Should check import for", name);
    end
    if (not self.toImport) then
        return;
    end
    if (not self.toImport[name]) then
        if (debugEnabled) then
            self:Debug("Nothing to import");
        end
        return;
    end
    if (debugEnabled) then
        self:Debug("Got hit in import data for", name);
    end

    local pdata= self:GetLegacyPlayerData();
    if (pdata == nil) then
        return;
    end

    local legacyData = {};

    for level, leveldata in pairs(pdata) do
        if (type(leveldata) == "table") then
            local kills = leveldata[name];
            if ((type(kills) == "number") and (kills > 0)) then
                legacyData[level] = kills;
            end
            leveldata[name] = nil;
            if (next(leveldata) == nil) then
                pdata[level] = nil;
            end
        end
    end

    if (next(legacyData) ~= nil) then
        if (debugEnabled) then
            self:Debug("Packaged legacy data for", name);
        end
        return legacyData;
    end

    if (debugEnabled) then
        self:Debug("No legacy data for", name);
    end
end