-- KillCounterTooltip
--
-- Code to manage tooltip augmentation
--
-- Globals: KillCounter, GameTooltip, UnitGUID, UnitName, UnitLevel

local KC = KillCounter;

local function FormatSummaryLine(label, total, level, session)
    if (session == nil) then
        if (total == level) then
            return string.format("%s: %d", label, total);
        end

        return string.format("%s: %d, %d this level", label, total, level);
    end

    if (session == level) then
        if (total == level) then
            return string.format("%s: %d", label, total);
        end

        return string.format("%s: %d, %d recently", label, total, session);
    end

    return string.format("%s: %d, %d recently, %d this level", label, total,
                     session, level);
end

KC.TOOLTIP_COLOR = { 0.6, 0.8, 1.0};

-- Cannot be local since event handler needs it.
function KC:GameTooltip_OnSetUnit(frame, unit)
    local options = self.options;

    if (options.NoTooltip) then
        return;
    end

    if (not UnitExists(unit)) then
        return;
    end

    local color = self.TOOLTIP_COLOR;

    local guid = UnitGUID(unit);
    local name = UnitName(unit);

    if (options.ShowGuid) then
        local colorguid = guid;
        local pre,uid,post = guid:match("^(0x..[34])(.......)(.*)$");
        if (pre) then
            colorguid = "|cff88ffff" .. pre
                .. "|cffffff00" .. uid
                .. "|r" .. post;
        end

        frame:AddLine("GUID: " .. colorguid, unpack(color));
    end

    local handle = self:GetHandle(guid);
    if (not handle) then
        frame:Show();
        return;
    end

    --frame:AddLine("KCHandle: " .. handle, unpack(color));

    local record = self:GetRecord(handle, false, name);
    if (not record) then
        if (not options.TerseTooltip) then
            frame:AddLine("No KillCounter data", unpack(color));
        end
        frame:Show();
        return;
    end

    local summ = record:GetSummary(UnitLevel("player"));
    local nSumm = #summ;
    local firstIdx = 1;
    if (nSumm > 0 and summ[1][1] == "Kills") then
        frame:AddLine(FormatSummaryLine(unpack(summ[1])), unpack(color));
        firstIdx = 2;
        if (nSumm == 2) then
            firstIdx = 3;
            frame:AddLine("(All " .. summ[2][1] .. ")",
                          unpack(color));
        end
    end
    if (not options.TerseTooltip) then
        for k = firstIdx, nSumm do
            local label, total, level, session = unpack(summ[k]);
            frame:AddLine(FormatSummaryLine("    " .. label,
                                            total, level, session), unpack(color));
        end
    end
    frame:Show();
end

local function InstallHandler(frame, handler, func)
    local cur = frame:GetScript(handler);
    if (cur ~= nil) then
        frame:HookScript(handler, func);
    else
        frame:SetScript(handler, func);
    end
end

InstallHandler(GameTooltip, "OnTooltipSetUnit",
               function(frame)
                   local name, unit = frame:GetUnit();
                   KC:GameTooltip_OnSetUnit(frame, unit)
               end);

